# RFC 3986 2.2.  Reserved Characters
#
#   reserved    = gen-delims / sub-delims
#
#   gen-delims  = ":" / "/" / "?" / "#" / "[" / "]" / "@"
#
#   sub-delims  = "!" / "$" / "&" / "'" / "(" / ")"
#               / "*" / "+" / "," / ";" / "="
#
GEN_DELIMS = ":/?#[]@"
SUB_DELIMS = "!$&'()*+,;="
RESERVED = GEN_DELIMS + SUB_DELIMS

# RFC 3986 2.3.  Unreserved Characters
#
#   unreserved  = ALPHA / DIGIT / "-" / "." / "_" / "~"
#
UNRESERVED = (
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ" "abcdefghijklmnopqrstuvwxyz" "0123456789" "-._~"
)



def get_version():
    import configparser
    import pathlib

    cp = configparser.ConfigParser()
    # Python 3.5 ConfigParser does not accept Path as filename
    cp.read(str(pathlib.Path(__file__).parent.parent / "setup.cfg"))
    return cp["metadata"]["version"]


project = "uritools"
copyright = "2014-2021 Thomas Kemmer"
version = get_version()
release = version

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.coverage",
    "sphinx.ext.doctest",
    "sphinx.ext.todo",
]
exclude_patterns = ["_build"]
master_doc = "index"
html_theme = "default"



import collections

from .encoding import uridecode


class DefragResult(collections.namedtuple("DefragResult", "uri fragment")):
    """Class to hold :func:`uridefrag` results."""

    __slots__ = ()  # prevent creation of instance dictionary

    def geturi(self):
        """Return the recombined version of the original URI as a string."""
        fragment = self.fragment
        if fragment is None:
            return self.uri
        elif isinstance(fragment, bytes):
            return self.uri + b"#" + fragment
        else:
            return self.uri + "#" + fragment

    def getfragment(self, default=None, encoding="utf-8", errors="strict"):
        """Return the decoded fragment identifier, or `default` if the
        original URI did not contain a fragment component.

        """
        fragment = self.fragment
        if fragment is not None:
            return uridecode(fragment, encoding, errors)
        else:
            return default


def uridefrag(uristring):
    """Remove an existing fragment component from a URI reference string."""
    if isinstance(uristring, bytes):
        parts = uristring.partition(b"#")
    else:
        parts = uristring.partition("#")
    return DefragResult(parts[0], parts[2] if parts[1] else None)



from .split import urisplit

# TODO: use specialized checks/regexes for performance


def isuri(uristring):
    """Return :const:`True` if `uristring` is a URI."""
    return urisplit(uristring).isuri()


def isabsuri(uristring):
    """Return :const:`True` if `uristring` is an absolute URI."""
    return urisplit(uristring).isabsuri()


def isnetpath(uristring):
    """Return :const:`True` if `uristring` is a network-path reference."""
    return urisplit(uristring).isnetpath()


def isabspath(uristring):
    """Return :const:`True` if `uristring` is an absolute-path reference."""
    return urisplit(uristring).isabspath()


def isrelpath(uristring):
    """Return :const:`True` if `uristring` is a relative-path reference."""
    return urisplit(uristring).isrelpath()


def issamedoc(uristring):
    """Return :const:`True` if `uristring` is a same-document reference."""
    return urisplit(uristring).issamedoc()



"""RFC 3986 compliant, scheme-agnostic replacement for `urllib.parse`.

This module defines RFC 3986 compliant replacements for the most
commonly used functions of the Python Standard Library
:mod:`urllib.parse` module.

"""

from .chars import GEN_DELIMS, RESERVED, SUB_DELIMS, UNRESERVED
from .classify import isabspath, isabsuri, isnetpath, isrelpath, issamedoc, isuri
from .compose import uricompose
from .defrag import DefragResult, uridefrag
from .encoding import uridecode, uriencode
from .join import urijoin
from .split import SplitResult, urisplit, uriunsplit

__all__ = (
    "GEN_DELIMS",
    "RESERVED",
    "SUB_DELIMS",
    "UNRESERVED",
    "DefragResult",
    "SplitResult",
    "isabspath",
    "isabsuri",
    "isnetpath",
    "isrelpath",
    "issamedoc",
    "isuri",
    "uricompose",
    "uridecode",
    "uridefrag",
    "uriencode",
    "urijoin",
    "urisplit",
    "uriunsplit",
)

__version__ = "3.0.2"



import unittest

from uritools import urisplit


class SplitTest(unittest.TestCase):
    def check(self, uri, parts, decoded=None):
        result = urisplit(uri)
        self.assertEqual(result, parts, "Error parsing %r" % uri)
        self.assertEqual(result.geturi(), uri, "Error recomposing %r" % uri)

    def test_rfc3986(self):
        """urisplit test cases from [RFC3986] 3. Syntax Components"""
        cases = [
            (
                "foo://example.com:8042/over/there?name=ferret#nose",
                ("foo", "example.com:8042", "/over/there", "name=ferret", "nose"),
            ),
            (
                "urn:example:animal:ferret:nose",
                ("urn", None, "example:animal:ferret:nose", None, None),
            ),
            (
                b"foo://example.com:8042/over/there?name=ferret#nose",
                (b"foo", b"example.com:8042", b"/over/there", b"name=ferret", b"nose"),
            ),
            (
                b"urn:example:animal:ferret:nose",
                (b"urn", None, b"example:animal:ferret:nose", None, None),
            ),
        ]
        for uri, parts in cases:
            self.check(uri, parts)

    def test_abnormal(self):
        cases = [
            ("", (None, None, "", None, None)),
            (":", (None, None, ":", None, None)),
            (":/", (None, None, ":/", None, None)),
            ("://", (None, None, "://", None, None)),
            ("://?", (None, None, "://", "", None)),
            ("://#", (None, None, "://", None, "")),
            ("://?#", (None, None, "://", "", "")),
            ("//", (None, "", "", None, None)),
            ("//:", (None, ":", "", None, None)),
            ("///", (None, "", "/", None, None)),
            ("//?", (None, "", "", "", None)),
            ("//#", (None, "", "", None, "")),
            ("//?#", (None, "", "", "", "")),
            ("?", (None, None, "", "", None)),
            ("??", (None, None, "", "?", None)),
            ("?#", (None, None, "", "", "")),
            ("#", (None, None, "", None, "")),
            ("##", (None, None, "", None, "#")),
            ("?:", (None, None, "", ":", None)),
            ("#:", (None, None, "", None, ":")),
            ("_:", (None, None, "_:", None, None)),
            (b"", (None, None, b"", None, None)),
            (b":", (None, None, b":", None, None)),
            (b":/", (None, None, b":/", None, None)),
            (b"://", (None, None, b"://", None, None)),
            (b"://?", (None, None, b"://", b"", None)),
            (b"://#", (None, None, b"://", None, b"")),
            (b"://?#", (None, None, b"://", b"", b"")),
            (b"//", (None, b"", b"", None, None)),
            (b"//:", (None, b":", b"", None, None)),
            (b"///", (None, b"", b"/", None, None)),
            (b"//?", (None, b"", b"", b"", None)),
            (b"//#", (None, b"", b"", None, b"")),
            (b"//?#", (None, b"", b"", b"", b"")),
            (b"?", (None, None, b"", b"", None)),
            (b"??", (None, None, b"", b"?", None)),
            (b"?#", (None, None, b"", b"", b"")),
            (b"#", (None, None, b"", None, b"")),
            (b"##", (None, None, b"", None, b"#")),
            (b"?:", (None, None, b"", b":", None)),
            (b"#:", (None, None, b"", None, b":")),
            (b"_:", (None, None, b"_:", None, None)),
        ]
        for uri, parts in cases:
            self.check(uri, parts)

    def test_members(self):
        uri = "foo://user@example.com:8042/over/there?name=ferret#nose"
        result = urisplit(uri)
        self.assertEqual(result.scheme, "foo")
        self.assertEqual(result.authority, "user@example.com:8042")
        self.assertEqual(result.path, "/over/there")
        self.assertEqual(result.query, "name=ferret")
        self.assertEqual(result.fragment, "nose")
        self.assertEqual(result.userinfo, "user")
        self.assertEqual(result.host, "example.com")
        self.assertEqual(result.port, "8042")
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), "foo")
        self.assertEqual(result.getauthority(), ("user", "example.com", 8042))
        self.assertEqual(result.getuserinfo(), "user")
        self.assertEqual(result.gethost(), "example.com")
        self.assertEqual(result.getport(), 8042)
        self.assertEqual(result.getpath(), "/over/there")
        self.assertEqual(result.getquery(), "name=ferret")
        self.assertEqual(dict(result.getquerydict()), {"name": ["ferret"]})
        self.assertEqual(list(result.getquerylist()), [("name", "ferret")])
        self.assertEqual(result.getfragment(), "nose")

        uri = "urn:example:animal:ferret:nose"
        result = urisplit(uri)
        self.assertEqual(result.scheme, "urn")
        self.assertEqual(result.authority, None)
        self.assertEqual(result.path, "example:animal:ferret:nose")
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, None)
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), "urn")
        self.assertEqual(result.getauthority(), (None, None, None))
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), None)
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), "example:animal:ferret:nose")
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

        uri = "file:///"
        result = urisplit(uri)
        self.assertEqual(result.scheme, "file")
        self.assertEqual(result.authority, "")
        self.assertEqual(result.path, "/")
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, "")
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), "file")
        self.assertEqual(result.getauthority(), (None, "", None))
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), "")
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), "/")
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

        uri = b"foo://user@example.com:8042/over/there?name=ferret#nose"
        result = urisplit(uri)
        self.assertEqual(result.scheme, b"foo")
        self.assertEqual(result.authority, b"user@example.com:8042")
        self.assertEqual(result.path, b"/over/there")
        self.assertEqual(result.query, b"name=ferret")
        self.assertEqual(result.fragment, b"nose")
        self.assertEqual(result.userinfo, b"user")
        self.assertEqual(result.host, b"example.com")
        self.assertEqual(result.port, b"8042")
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), "foo")
        self.assertEqual(result.getauthority(), ("user", "example.com", 8042))
        self.assertEqual(result.getuserinfo(), "user")
        self.assertEqual(result.gethost(), "example.com")
        self.assertEqual(result.getport(), 8042)
        self.assertEqual(result.getpath(), "/over/there")
        self.assertEqual(result.getquery(), "name=ferret")
        self.assertEqual(dict(result.getquerydict()), {"name": ["ferret"]})
        self.assertEqual(list(result.getquerylist()), [("name", "ferret")])
        self.assertEqual(result.getfragment(), "nose")

        uri = b"urn:example:animal:ferret:nose"
        result = urisplit(uri)
        self.assertEqual(result.scheme, b"urn")
        self.assertEqual(result.authority, None)
        self.assertEqual(result.path, b"example:animal:ferret:nose")
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, None)
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), "urn")
        self.assertEqual(result.getauthority(), (None, None, None))
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), None)
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), "example:animal:ferret:nose")
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

        uri = b"file:///"
        result = urisplit(uri)
        self.assertEqual(result.scheme, b"file")
        self.assertEqual(result.authority, b"")
        self.assertEqual(result.path, b"/")
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, b"")
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), "file")
        self.assertEqual(result.getauthority(), (None, "", None))
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), "")
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), "/")
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

    def test_getscheme(self):
        self.assertEqual(urisplit("foo").getscheme(default="bar"), "bar")
        self.assertEqual(urisplit("foo:").getscheme(default="bar"), "foo")
        self.assertEqual(urisplit("FOO:").getscheme(default="bar"), "foo")
        self.assertEqual(urisplit("FOO_BAR:/").getscheme(default="x"), "x")
        self.assertEqual(urisplit(b"foo").getscheme(default="bar"), "bar")
        self.assertEqual(urisplit(b"foo:").getscheme(default="bar"), "foo")
        self.assertEqual(urisplit(b"FOO:").getscheme(default="bar"), "foo")
        self.assertEqual(urisplit(b"FOO_BAR:/").getscheme(default="x"), "x")

    def test_getauthority(self):
        from ipaddress import IPv4Address, IPv6Address

        cases = [
            ("urn:example:animal:ferret:nose", None, (None, None, None)),
            ("file:///", None, (None, "", None)),
            (
                "http://userinfo@Test.python.org:5432/foo/",
                None,
                ("userinfo", "test.python.org", 5432),
            ),
            (
                "http://userinfo@12.34.56.78:5432/foo/",
                None,
                ("userinfo", IPv4Address("12.34.56.78"), 5432),
            ),
            (
                "http://userinfo@[::1]:5432/foo/",
                None,
                ("userinfo", IPv6Address("::1"), 5432),
            ),
            (
                "urn:example:animal:ferret:nose",
                ("nobody", "localhost", 42),
                ("nobody", "localhost", 42),
            ),
            ("file:///", ("nobody", "localhost", 42), ("nobody", "localhost", 42)),
            (
                "http://Test.python.org/foo/",
                ("nobody", "localhost", 42),
                ("nobody", "test.python.org", 42),
            ),
            (
                "http://userinfo@Test.python.org/foo/",
                ("nobody", "localhost", 42),
                ("userinfo", "test.python.org", 42),
            ),
            (
                "http://Test.python.org:5432/foo/",
                ("nobody", "localhost", 42),
                ("nobody", "test.python.org", 5432),
            ),
            (
                "http://userinfo@Test.python.org:5432/foo/",
                ("nobody", "localhost", 42),
                ("userinfo", "test.python.org", 5432),
            ),
        ]
        for uri, default, authority in cases:
            self.assertEqual(urisplit(uri).getauthority(default), authority)
        for uri in ["http://[::1/", "http://::1]/"]:
            with self.assertRaises(ValueError, msg="%r" % uri):
                urisplit(uri).getauthority()
            with self.assertRaises(ValueError, msg="%r" % uri):
                urisplit(uri.encode()).getauthority()
        with self.assertRaises(TypeError, msg="%r" % uri):
            urisplit("").getauthority(42)
        with self.assertRaises(ValueError, msg="%r" % uri):
            urisplit("").getauthority(("userinfo", "test.python.org"))

    def test_gethost(self):
        from ipaddress import IPv4Address, IPv6Address

        cases = [
            ("http://Test.python.org:5432/foo/", "test.python.org"),
            ("http://12.34.56.78:5432/foo/", IPv4Address("12.34.56.78")),
            ("http://[::1]:5432/foo/", IPv6Address("::1")),
        ]
        for uri, host in cases:
            self.assertEqual(urisplit(uri).gethost(), host)
            self.assertEqual(urisplit(uri.encode()).gethost(), host)
        for uri in ["http://[::1/", "http://::1]/"]:
            with self.assertRaises(ValueError, msg="%r" % uri):
                urisplit(uri).gethost()
            with self.assertRaises(ValueError, msg="%r" % uri):
                urisplit(uri.encode()).gethost()

    def test_getport(self):
        for uri in ["foo://bar", "foo://bar:", "foo://bar/", "foo://bar:/"]:
            result = urisplit(uri)
            if result.authority.endswith(":"):
                self.assertEqual(result.port, "")
            else:
                self.assertEqual(result.port, None)
            self.assertEqual(result.gethost(), "bar")
            self.assertEqual(result.getport(8000), 8000)

    def test_getpath(self):
        cases = [
            ("", "", "/"),
            (".", "./", "/"),
            ("./", "./", "/"),
            ("./.", "./", "/"),
            ("./..", "../", "/"),
            ("./foo", "foo", "/foo"),
            ("./foo/", "foo/", "/foo/"),
            ("./foo/.", "foo/", "/foo/"),
            ("./foo/..", "./", "/"),
            ("..", "../", "/"),
            ("../", "../", "/"),
            ("../.", "../", "/"),
            ("../..", "../../", "/"),
            ("../foo", "../foo", "/foo"),
            ("../foo/", "../foo/", "/foo/"),
            ("../foo/.", "../foo/", "/foo/"),
            ("../foo/..", "../", "/"),
            ("../../foo", "../../foo", "/foo"),
            ("../../foo/", "../../foo/", "/foo/"),
            ("../../foo/.", "../../foo/", "/foo/"),
            ("../../foo/..", "../../", "/"),
            ("../../foo/../bar", "../../bar", "/bar"),
            ("../../foo/../bar/", "../../bar/", "/bar/"),
            ("../../foo/../bar/.", "../../bar/", "/bar/"),
            ("../../foo/../bar/..", "../../", "/"),
            ("../../foo/../..", "../../../", "/"),
        ]
        for uri, relpath, abspath in cases:
            parts = urisplit(uri)
            self.assertEqual(relpath, parts.getpath())
            parts = urisplit(uri.encode("ascii"))
            self.assertEqual(relpath, parts.getpath())
            parts = urisplit("/" + uri)
            self.assertEqual(abspath, parts.getpath())
            parts = urisplit(("/" + uri).encode("ascii"))
            self.assertEqual(abspath, parts.getpath())

    def test_getquery(self):
        cases = [
            ("", [], {}),
            ("?", [], {}),
            ("?&", [], {}),
            ("?&&", [], {}),
            ("?=", [("", "")], {"": [""]}),
            ("?=a", [("", "a")], {"": ["a"]}),
            ("?a", [("a", None)], {"a": [None]}),
            ("?a=", [("a", "")], {"a": [""]}),
            ("?&a=b", [("a", "b")], {"a": ["b"]}),
            (
                "?a=a+b&b=b+c",
                [("a", "a+b"), ("b", "b+c")],
                {"a": ["a+b"], "b": ["b+c"]},
            ),
            (
                "?a=a%20b&b=b%20c",
                [("a", "a b"), ("b", "b c")],
                {"a": ["a b"], "b": ["b c"]},
            ),
            ("?a=1&a=2", [("a", "1"), ("a", "2")], {"a": ["1", "2"]}),
        ]
        for query, querylist, querydict in cases:
            parts = urisplit(query)
            self.assertEqual(
                parts.getquerylist(),
                querylist,
                "Error parsing query dict for %r" % query,
            )
            self.assertEqual(
                parts.getquerydict(),
                querydict,
                "Error parsing query list for %r" % query,
            )

    def test_getquerysep(self):
        cases = [
            ("&", "?a=b", [("a", "b")]),
            (";", "?a=b", [("a", "b")]),
            ("&", "?a=a+b&b=b+c", [("a", "a+b"), ("b", "b+c")]),
            (";", "?a=a+b;b=b+c", [("a", "a+b"), ("b", "b+c")]),
            ("&", "?a=a+b;b=b+c", [("a", "a+b;b=b+c")]),
            (";", "?a=a+b&b=b+c", [("a", "a+b&b=b+c")]),
            ("&", "?a&b", [("a", None), ("b", None)]),
            (";", "?a;b", [("a", None), ("b", None)]),
            (b"&", "?a&b", [("a", None), ("b", None)]),
            ("&", b"?a&b", [("a", None), ("b", None)]),
        ]
        for sep, query, querylist in cases:
            parts = urisplit(query)
            self.assertEqual(
                parts.getquerylist(sep),
                querylist,
                "Error parsing query list for %r" % query,
            )

    def test_ip_literal(self):
        cases = [
            ("http://Test.python.org:5432/foo/", "test.python.org", 5432),
            ("http://12.34.56.78:5432/foo/", "12.34.56.78", 5432),
            ("http://[::1]:5432/foo/", "::1", 5432),
            ("http://[dead:beef::1]:5432/foo/", "dead:beef::1", 5432),
            ("http://[dead:beef::]:5432/foo/", "dead:beef::", 5432),
            (
                "http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]:5432/foo/",
                "dead:beef:cafe:5417:affe:8fa3:deaf:feed",
                5432,
            ),
            ("http://[::12.34.56.78]:5432/foo/", "::c22:384e", 5432),
            ("http://[::ffff:12.34.56.78]:5432/foo/", "::ffff:c22:384e", 5432),
            ("http://Test.python.org/foo/", "test.python.org", None),
            ("http://12.34.56.78/foo/", "12.34.56.78", None),
            ("http://[::1]/foo/", "::1", None),
            ("http://[dead:beef::1]/foo/", "dead:beef::1", None),
            ("http://[dead:beef::]/foo/", "dead:beef::", None),
            (
                "http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]/foo/",
                "dead:beef:cafe:5417:affe:8fa3:deaf:feed",
                None,
            ),
            ("http://[::12.34.56.78]/foo/", "::c22:384e", None),
            ("http://[::ffff:12.34.56.78]/foo/", "::ffff:c22:384e", None),
            ("http://Test.python.org:/foo/", "test.python.org", None),
            ("http://12.34.56.78:/foo/", "12.34.56.78", None),
            ("http://[::1]:/foo/", "::1", None),
            ("http://[dead:beef::1]:/foo/", "dead:beef::1", None),
            ("http://[dead:beef::]:/foo/", "dead:beef::", None),
            (
                "http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]:/foo/",
                "dead:beef:cafe:5417:affe:8fa3:deaf:feed",
                None,
            ),
            ("http://[::12.34.56.78]:/foo/", "::c22:384e", None),
            ("http://[::ffff:12.34.56.78]:/foo/", "::ffff:c22:384e", None),
        ]
        for uri, host, port in cases:
            parts = urisplit(uri)
            self.assertEqual(host, str(parts.gethost()))
            self.assertEqual(port, parts.getport())
            parts = urisplit(uri.encode("ascii"))
            self.assertEqual(host, str(parts.gethost()))
            self.assertEqual(port, parts.getport())

    def test_invalid_ip_literal(self):
        uris = [
            "http://::12.34.56.78]/",
            "http://[::1/foo/",
            "ftp://[::1/foo/bad]/bad",
            "http://[::1/foo/bad]/bad",
            "http://[foo]/",
            "http://[v7.future]",
        ]
        for uri in uris:
            with self.assertRaises(ValueError, msg="%r" % uri):
                urisplit(uri).gethost()
            with self.assertRaises(ValueError, msg="%r" % uri.encode("ascii")):
                urisplit(uri.encode("ascii")).gethost()



from string import hexdigits as _hex

from .chars import UNRESERVED


# RFC 3986 2.1: For consistency, URI producers and normalizers should
# use uppercase hexadecimal digits for all percent-encodings.
def _pctenc(byte):
    return ("%%%02X" % byte).encode()


_unreserved = frozenset(UNRESERVED.encode())

_encoded = {b"": [bytes([i]) if i in _unreserved else _pctenc(i) for i in range(256)]}

_decoded = {(a + b).encode(): bytes.fromhex(a + b) for a in _hex for b in _hex}


def uriencode(uristring, safe="", encoding="utf-8", errors="strict"):
    """Encode a URI string or string component."""
    if not isinstance(uristring, bytes):
        uristring = uristring.encode(encoding, errors)
    if not isinstance(safe, bytes):
        safe = safe.encode("ascii")
    try:
        encoded = _encoded[safe]
    except KeyError:
        encoded = _encoded[b""][:]
        for i in safe:
            encoded[i] = bytes([i])
        _encoded[safe] = encoded
    return b"".join(map(encoded.__getitem__, uristring))


def uridecode(uristring, encoding="utf-8", errors="strict"):
    """Decode a URI string or string component."""
    if not isinstance(uristring, bytes):
        uristring = uristring.encode(encoding or "ascii", errors)
    parts = uristring.split(b"%")
    result = [parts[0]]
    append = result.append
    decode = _decoded.get
    for s in parts[1:]:
        append(decode(s[:2], b"%" + s[:2]))
        append(s[2:])
    if encoding is not None:
        return b"".join(result).decode(encoding, errors)
    else:
        return b"".join(result)
